@@ -271,8 +271,8 @@ class AuthController():
         if self.request.matched_route.name == 'ppss:user:editself':
             userid = self.request.session[Conf.sessionuser]['id']
             user = self.getUserById(userid)
-            self.userid = userid
-            self.user = user
+            #self.userid = userid
+            #self.user = user
         l.debug("***{id} -> {user}".format(user=user,id=userid))
         retdict = dict(self.retdict,**{'msg':"",'res':True,'userid':userid} )
         if not user:
@@ -293,24 +293,40 @@ class AuthController():
         retdict.update({"user" : user, 'allgroups':editablegroups })
 
         if self.request.POST:
+            username = user.username
             if userid<0:
                 self.request.dbsession.add(user)
+                user.username = self.request.params.get("username",user.username)
             if not user:
                 return retdict
             newpassword = self.request.params.get("password","")
             if newpassword:
-                l.info("*****changing password for {}".format(user.username) )
-                l.info("checkPassword result = {res}".format(res = checkPassword(user,newpassword) ))
-                user.setPassword(newpassword)
-            user.username = self.request.params.get("username",user.username)
+                
+                confirmnewpassword = self.request.params.get("confirmnewpassword","")
+                currentpassword = self.request.params.get("currentpassword","")
+
+                #existing user must match older password
+                if userid>=0 and (not PPSsuser.checkLogin(username,currentpassword,self.request.dbsession)): 
+                    retdict['msg'] = "your current password does not match" 
+
+                elif newpassword==confirmnewpassword:
+                    l.info("*****changing password for {}".format(user.username) )
+                    res = checkPassword(user,newpassword)
+                    l.info("checkPassword result for user {} = {}".format(user.username, res ))
+                    if res:
+                        user.setPassword(newpassword)
+                    else:
+                        retdict['msg'] = "new password doesn't match constraints." 
+                else:
+                    retdict['msg'] = "new password doesn't match confirmation field." 
             user.enabled = 1 if self.request.params.get("enabled")=="1" else 0
             
             groups=map(int,self.request.params.getall("allgroups"))
             l.debug("group={groups}".format(groups=groups ))
-            usergroups = [PPSsgroup.byId(groupid,self.request.dbsession) for groupid in groups]
+            usergroups = [PPSsgroup.byId(groupid,self.request.dbsession) for groupid in groups if groupid in set([g.id for g in editablegroups ])]
             user.groups = usergroups
             self.request.dbsession.flush()
-            return HTTPFound(self.request.route_url('ppss:user:edit',elementid = user.id) )
+            #return HTTPFound(self.request.route_url('ppss:user:edit',elementid = user.id) )
             #return retdict
 
         return retdict
